///
/// \file    HardwareDisplay.h
/// \brief   Map channel addresses to (x,y) coordinate for display
/// \author  messier@indiana.edu
/// \version $Id: HardwareDisplay.h,v 1.1 2013/07/19 02:36:43 messier Exp $
///
#ifndef _HARDWAREDISPLAY_H
#define _HARDWAREDISPLAY_H

namespace daqchannelmap {
  ///
  /// Map hardware addresses to a set of x,y pairs for display
  ///
  class HardwareDisplay {
  public:
    ///.................................................................
    /// Configure the mapping for a detector
    /// \param det : Detector ID following NovaDAQConventions
    ///
    void SetupDet(int det);
    void SetupNDOS();
    void SetupNearDet();
    void SetupFarDet();
    
    ///.................................................................
    /// Assign an x,y display location to a DAQ hardware address
    ///
    /// \param db - diblock number 1...14
    /// \param dcm - DCM number 1...12
    /// \param feb - FEB number 0...63
    /// \param pix - pixel number 0...31
    /// \param ix - On return, the x display location assigned
    /// \param iy - On return, the y display location assigned
    //
    void DAQToXY(unsigned int  db,
		 unsigned int  dcm,
		 unsigned int  feb,
		 unsigned int  pix,
		 unsigned int* ix,
		 unsigned int* iy);

    ///.................................................................
    /// Map an x,y location on the display to a DAQ hardware address
    /// \param db - on return, the diblock address
    /// \param dcm - on return, the DCM address
    /// \param feb - on return, the FEB address
    /// \param pix - on return, the pixel address
    /// \param ix  - input x display location
    /// \param iy  - input y display location
    ///
    void XYToDAQ(unsigned int* db,
		 unsigned int* dcm,
		 unsigned int* feb,
		 unsigned int* pix,
		 unsigned int  ix,
		 unsigned int  iy);

    ///.................................................................
    /// Convert a DAQ address to a global plane / module / global cell
    /// address
    ///
    /// \param block  - on return, block number 0...27
    /// \param plane  - on return, global plane number (0...nblock*32)
    /// \param view   - on return, X measuring (vertical) or Y measuring (horizontal)
    /// \param module - on return, Module number 0...11
    /// \param cell   - on return, Global cell number 0...nmodule*32
    /// \param db     - input DAQ diblock address
    /// \param dcm    - input DAQ DCM address
    /// \param feb    - input DAQ FEB address
    /// \param pix    - input DAQ APD pixel address
    ///
    void DAQToPlaneModCell(unsigned int* block,
			   unsigned int* plane,
			   unsigned int* view,
			   unsigned int* module,
			   unsigned int* cell,
			   unsigned int  db,
			   unsigned int  dcm,
			   unsigned int  feb,
			   unsigned int  pix);

    ///.................................................................
    /// Convert a DAQ address to a plane number within a block /
    /// module / global cell address
    ///
    /// \param block  - on return, block number 0...27
    /// \param pinb   - on return, plane number within a block (0...31)
    /// \param view   - on return, X measuring (vertical) or Y measuring (horizontal)
    /// \param module - on return, Module number 0...11
    /// \param cell   - on return, Global cell number 0...nmodule*32
    /// \param db     - input DAQ diblock address
    /// \param dcm    - input DAQ DCM address
    /// \param feb    - input DAQ FEB address
    /// \param pix    - input DAQ APD pixel address
    ///
    void DAQToPlaneInBlockModCell(unsigned int* block,
				  unsigned int* pinb,
				  unsigned int* view,
				  unsigned int* module,
				  unsigned int* cell,
				  unsigned int  db,
				  unsigned int  dcm,
				  unsigned int  feb,
				  unsigned int  pix);

    ///.................................................................
    /// Convert a global plane and cell address to a DAQ address
    ///
    /// \param plane  - input global plane number, 0...nblock*32
    /// \param pinb   - on return, plane number within a block (0...31)
    /// \param view   - on return, X measuring (vertical) or Y measuring (horizontal)
    /// \param module - on return, Module number 0...11
    /// \param cell   - input global cell number 0...nmodule*32
    /// \param db     - input DAQ diblock address
    /// \param block  - on return block address
    /// \param dcm    - on return, DAQ DCM address
    /// \param feb    - on return, DAQ FEB address
    /// \param pix    - on return, DAQ APD pixel address
    ///
    void PlaneCellToDAQ(unsigned int  plane,
			unsigned int* pinb,
			unsigned int* view,
			unsigned int* module,
			unsigned int  cell,
			unsigned int* db,
			unsigned int* block,
			unsigned int* dcm,
			unsigned int* feb,
			unsigned int* pix);

    ///.................................................................
    /// Get the lower left hand corner on the display for a Diblock
    /// \param db : Diblock number, 1-14...
    /// \param ix : On return, x pixel location within the whole display
    /// \param iy : On return, y pixel location within the whole display
    ///
    void DiblockXY(unsigned int db,  unsigned int* ix, unsigned int* iy);
    
    ///.................................................................
    /// Get the lower left hand corner on the display for a DCM
    /// \param DCM : DCM number, 1...12
    /// \param ix  : On return, x pixel location within a Diblock block
    /// \param iy  : On return, y pixel location within a Diblock block
    ///
    void DCMXY(unsigned int dcm, unsigned int* ix, unsigned int* iy);

    ///.................................................................
    /// Get the lower left hand corner on the display for an FEB
    /// 
    /// \param FEB : FEB number, 0...63
    /// \param ix  : On return, x pixel location within a DCM block
    /// \param iy  : On return, y pixel location within a DCM block
    //
    void FEBXY(unsigned int feb, unsigned int* ix, unsigned int* iy);

    ///.................................................................
    /// Get the location of a pixel on the display inside an FEB
    /// 
    /// \param pix - Pixel number 0...31
    /// \param ix  - On return, x display location within an FEB
    /// \param iy  - On return, y display location within an FEB
    ///
    void PixXY(unsigned int pix, unsigned int* ix, unsigned int* iy);
    
    ///.................................................................
    /// Map an x display location to a diblock address
    /// \param db : on return, the diblock address (1...14)
    /// \param ix : input x display location
    ///
    void XYDiblock(unsigned int* db,  unsigned int ix);

    ///.................................................................
    /// Map a y display location to a diblock address
    /// \param dcm : on return, the DCM address (1...14)
    /// \param iy  : input y display location
    ///
    void XYDCM(unsigned int* dcm, unsigned int iy);
    
    ///.................................................................
    /// Map a x,y display location to a diblock address
    /// \param feb : on return, the FEB address (1...14)
    /// \param ix  : input x display location
    /// \param iy  : input y display location
    ///
    void XYFEB(unsigned int* feb, unsigned int ix, unsigned int iy);
    
    ///.................................................................
    /// Map an x,y display location to a pixel address
    /// \param pix : on return, the pixel address (1...14)
    /// \param ix  : input x display location
    /// \param iy  : input y display location
    ///
    void XYPix(unsigned int* pix, unsigned int ix, unsigned int iy);

  public:
    unsigned int fDetId;        ///< DAQ detector ID
    unsigned int fNdiblocks;    ///< Number of diblocks in detector
    unsigned int fNdcmTop;      ///< Number of DMC's per diblock on top of detector
    unsigned int fNdcmSide;     ///< Number of DMC's per diblock on side of detector
    unsigned int fNfebCols;     ///< Number of FEB columns in a DCM
    unsigned int fNfebRows;     ///< Number of FEB rows in a DCM
    unsigned int fNpixCols;     ///< Number of pixel columns in an FEB
    unsigned int fNpixRows;     ///< Number of pixel rows in an FEB
    unsigned int fNxPerDB;      ///< Width of 1 diblock
    unsigned int fNyPerDCM;     ///< Height of 1 DCM
    unsigned int fNplanes;      ///< Number of planes in the detector
    unsigned int fNplanesPerBL; ///< Number of planes per block
    unsigned int fNxcells;      ///< Width of detector in units of cells
    unsigned int fNycells;      ///< Height of detector in units of cells
    unsigned int fXmax;         ///< Width of entire view
    unsigned int fYmax;         ///< Height of entire view
    unsigned int fNcellPerMod;  ///< Number of cells per module
  };
}
#endif
////////////////////////////////////////////////////////////////////////
