/*
#include "DAQDataFormats/RawRunV0.h"
#include "DAQDataFormats/DataFormatException.h"
#include <boost/lexical_cast.hpp>
#include <cstdio>

using namespace daqdataformats;

//------------------------------------------------------------------------------
// Default constructor
RawRunV0::RawRunV0():
     _Header                    (NULL)
    ,_Tail                      (NULL)
    ,_ConfigurationBlock        (NULL)
    ,_FloatingEvent             (NULL)
    ,_auxEventHeader            (NULL)
    ,_FloatingEventNumber       (-1)
    ,_eventVersion              (rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
    ,_configurationBlockVersion (rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
{}

//------------------------------------------------------------------------------
// Destructor
RawRunV0::~RawRunV0() { }

//------------------------------------------------------------------------------
// Initialization of Data members
void RawRunV0::init1(){

    if(_shouldBufferBeInternal){
        resizeInternalBuffer(_Header->sizeofdata());
        /// Check pointers after each read or resize buffer
        checkBufferInternalness();
    }

    /// by default, init initializes RunSize to the size of the Header
    _Header->init();


    // Initialize the _FloatingEventNumber
    _FloatingEventNumber = -1;
}


//------------------------------------------------------------------------------
/// The very first time we run figureOutEventSize, we don't know Event Version,
/// So, we need to figure it out first
/// After that, we just figure out the size
uint32_t RawRunV0::figureOutEventSize1(void* event_buffer){

    if(_auxEventHeader) delete _auxEventHeader;

    _auxEventHeader = new RawEventHeader(event_buffer, _eventVersion);

    _eventVersion   = (version_t)_auxEventHeader->getVersion();

    /// Now, reassign the function pointer for this function
    /// to point to figureOutEventSizeWithoutFiguringOutVersion
    _function_figureOutEventSize = _function_figureOutEventSizeWithoutFiguringOutVersion;

    return _auxEventHeader->getEventSize();
}

//------------------------------------------------------------------------------
/// The very first time we run figureOutEventSize, we don't know Event Version,
/// So, we need to figure it out first
/// After that, we just figure out the size
uint32_t RawRunV0::figureOutEventSizeWithoutFiguringOutVersion1(void* event_buffer){

    _auxEventHeader->setBufferSource(event_buffer);

    return _auxEventHeader->getEventSize();
}

//------------------------------------------------------------------------------
// Add a Event into the Run changing the Header appropriately
bool RawRunV0::addEvent1(void* event_buffer){


    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 0");

    const uint32_t current_size = sizeofdata(); //remember the current buffer position
    const uint32_t event_size   = figureOutEventSize(event_buffer);
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 1");

    /// Read Event into the Buffer
    readDataGeneral(event_buffer, event_size, current_size);
    /// Check pointers after each read or resize buffer
    checkBufferInternalness();

    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 2");

    const uint32_t number_of_events_current = getTotalEvents();
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 3");

    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" setFloatingEvent "<<number_of_events_current);
    /// set the Floating event to be able to read Trigger Info
    setFloatingEvent(number_of_events_current);
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 4");

    // Setup the RunHeader Header
    // Advance the Event number in the Run
    _Header->setTotalEvents(number_of_events_current + 1);
    // Change the Run size
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 5");

    _Header->setRunSize(current_size + event_size);
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 6");

    // Adding Trigger information
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 7"<<_FloatingEvent);
    RawTrigger*             trigger        = _FloatingEvent->getTrigger();
    trigger->print();
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 8");
    const RawTriggerHeader* trigger_header = trigger->getTriggerHeader();
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 9");
    const RawTriggerMask*   trigger_mask   = trigger->getTriggerMask();
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 10");

    PRINT_ON_DEBUG("Adding event; trigger SIM = " << trigger_mask->getMonteCarloFlag())
            
           _Header->setTriggerVersion        (trigger_header->getVersion());
           _Header->setTriggerMC             (trigger_mask->getMonteCarloFlag());
           _Header->setVTMod                 (trigger_mask->getTrigMod());
           _Header->addValidTriggerTypesLow  (trigger_mask->getTrigMask_Low());
           _Header->addValidTriggerTypesHigh (trigger_mask->getTrigMask_High());
    return _Header->addValidTriggerTypesHigh2(trigger_mask->getTrigMask_High2());

}

//------------------------------------------------------------------------------
bool RawRunV0::close(){return close(true);}

//------------------------------------------------------------------------------
// Closing the Run by adding the Index (if available), tail and the CRC
bool RawRunV0::close1(bool produce_crc){

    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" 0")

    const uint32_t old_run_size = sizeofdata();

    const uint32_t run_header_size = _Header->sizeofdata();

    /// Add the size of the tail + CRC
    const uint32_t new_run_size = old_run_size + run_header_size + 1;

    
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" Old size "<<old_run_size)

    _Header->setRunSize(new_run_size);

    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" New size "<<new_run_size)

    /// In case Buffer is small
    if(_shouldBufferBeInternal)
        reserveInternalBuffer(new_run_size);

    /// Read RunHeader into the Buffer
    readDataGeneral(_Buffer, run_header_size, old_run_size);
    /// Check pointers after each read or resize buffer
    checkBufferInternalness();
    
    
    
    /// set Tail
    getTail()->setTail();

    /// At this point _Tail is legal
    _Header->close();
    _Tail->close();

    /// Finally, add CRC in the very end of _Buffer
    if(produce_crc){
        /// Run CRC
        addCheckSum();
    }

    return true;
}

//------------------------------------------------------------------------------
// This is a general method to read the event from millislice_buffer to _Buffer
void* RawRunV0::readData1(void* run_buffer){

    const uint32_t run_header_size = _Header->sizeofdata();

    /// Read the run Header
    run_buffer = readDataGeneral(run_buffer, run_header_size, 0);

    /// Check pointers after each read or resize buffer
    checkBufferInternalness();

    /// Read the rest of the run
    run_buffer = readDataGeneral(run_buffer, sizeofdata() - run_header_size, run_header_size);

    /// Check pointers after each read or resize buffer
    checkBufferInternalness();

    return run_buffer;
}


//------------------------------------------------------------------------------
//Getters
RawRunHeader*          RawRunV0::getHeader1()             { return _Header; }
RawEvent*              RawRunV0::getFloatingEvent1()      { return _FloatingEvent; }
RawConfigurationBlock* RawRunV0::getConfigurationBlock1() { return _ConfigurationBlock; }

//------------------------------------------------------------------------------
// Assuming the run is closed. Don't use it on non-closed runs!
RawRunTail* RawRunV0::getTail1(){
    uint32_t* pointer = &RAWBUFF32[sizeofdata() - _Header->sizeofdata() - 1];

    if(!_Tail){
        _Tail = new RawRunTail(pointer, _version);
    }
    else{
        _Tail->setBufferSource(pointer);
    }
    return _Tail;
}

//------------------------------------------------------------------------------
//Getters
RawRunHeader*          RawRunV0::getHeaderConst1()             const { return _Header; }
RawEvent*              RawRunV0::getFloatingEventConst1()      const { return _FloatingEvent; }
RawConfigurationBlock* RawRunV0::getConfigurationBlockConst1() const { return _ConfigurationBlock; }
RawRunTail*            RawRunV0::getTailConst1()               const { 

    uint32_t* pointer = &RAWBUFF32[sizeofdata() - _Header->sizeofdata() - 1];

    if(!_Tail){
        _Tail = new RawRunTail(pointer, _version);
    }
    else{
        _Tail->setBufferSource(pointer);
    }
    return _Tail;
}


/// Getters
uint32_t RawRunV0::getVersion1() const { return _Header->getVersion();}
uint32_t RawRunV0::getMarkerHi1() const { return _Header->getMarkerHi();}
uint32_t RawRunV0::getMarkerLo1() const { return _Header->getMarkerLo();}
uint32_t RawRunV0::getRunNumber1() const { return _Header->getRunNumber();}
uint32_t RawRunV0::getRunType1() const { return _Header->getRunType();}
uint32_t RawRunV0::getConfigIDX1() const { return _Header->getConfigIDX();}
uint32_t RawRunV0::getPartition1() const { return _Header->getPartition();}
uint32_t RawRunV0::getSubrun1() const { return _Header->getSubrun();}
uint32_t RawRunV0::getConfigurationBlockSet1() const { return _Header->getConfigurationBlockSet();}
uint32_t RawRunV0::getRunControlVersion1() const { return _Header->getRunControlVersion();}
uint32_t RawRunV0::getRunControlID1() const { return _Header->getRunControlID();}
uint32_t RawRunV0::getDataLoggerVersion1() const { return _Header->getDataLoggerVersion();}
uint32_t RawRunV0::getDataLoggerID1() const { return _Header->getDataLoggerID();}
uint32_t RawRunV0::getRunStartTime1() const { return _Header->getRunStartTime();}
uint32_t RawRunV0::getSubRunStartTime1() const { return _Header->getSubRunStartTime();}
uint32_t RawRunV0::getSubRunEndTime1() const { return _Header->getSubRunEndTime ();}
uint32_t RawRunV0::getRunEndTime1() const { return _Header->getRunEndTime();}
uint32_t RawRunV0::getTotalEvents1() const { return _Header->getTotalEvents();}
uint32_t RawRunV0::getUniversalEventStartLo1() const { return _Header->getUniversalEventStartLo();}
uint32_t RawRunV0::getUniversalEventStartHi1() const { return _Header->getUniversalEventStartHi();}
uint64_t RawRunV0::getUniversalEventStart1() const { return _Header->getUniversalEventStartHi();}
uint32_t RawRunV0::getUniversalEventEndLo1() const { return _Header->getUniversalEventEndLo();}
uint32_t RawRunV0::getUniversalEventEndHi1() const { return _Header->getUniversalEventEndHi();}
uint64_t RawRunV0::getUniversalEventEnd1() const { return _Header->getUniversalEventEnd();}
uint32_t RawRunV0::getRunSize1() const { return _Header->getRunSize();}
uint32_t RawRunV0::getTriggerVersion1() const { return _Header->getTriggerVersion();}
uint32_t RawRunV0::getTriggerCtrlID1() const { return _Header->getTriggerCtrlID();}
uint32_t RawRunV0::getTriggerMC1() const { return _Header->getTriggerMC();}
uint32_t RawRunV0::getVTMod1() const { return _Header->getVTMod();}
uint32_t RawRunV0::getValidTriggerTypesLow1() const { return _Header->getValidTriggerTypesLow();}
uint32_t RawRunV0::getValidTriggerTypesHigh1() const { return _Header->getValidTriggerTypesHigh();}
uint32_t RawRunV0::getValidTriggerTypesHigh21() const { return _Header->getValidTriggerTypesHigh2();}
uint32_t RawRunV0::getTriggerListIDX1() const { return _Header->getTriggerListIDX();}
uint32_t RawRunV0::getTriggerPrescaleListIDX1() const { return _Header->getTriggerPrescaleListIDX();}


bool RawRunV0::setConfigurationBlock(const RawConfigurationBlock&  copyin ){
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__);
    return setConfigurationBlock1(copyin);
}
bool RawRunV0::setConfigurationBlock(void* pointer){ 
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__);
    return setConfigurationBlock2(pointer);
}
//------------------------------------------------------------------------------
bool RawRunV0::setConfigurationBlock11 (const RawConfigurationBlock&  copyin){ 
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__);
    return setConfigurationBlock(copyin.getBuffer());
}

//------------------------------------------------------------------------------
bool RawRunV0::setConfigurationBlock21 (void*  pointer){


    _configurationBlockVersion = RawConfigurationBlock::figureOutTheVersionStatic(pointer);

    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<"Conf block version "<<_configurationBlockVersion)

    const RawConfigurationHeader temp_config_block_header(pointer, _configurationBlockVersion);

    const uint32_t current_size      = sizeofdata(); //remember the current buffer position
    const uint32_t config_block_size = temp_config_block_header.getConfigurationBlockSize();

    /// Read Config Block into the Buffer
    readDataGeneral(pointer, config_block_size, current_size);
    /// Check pointers after each read or resize buffer
    checkBufferInternalness();



    if(_ConfigurationBlock){
        _ConfigurationBlock->setBufferSource(&RAWBUFF32[current_size]);
    }
    else{
        _ConfigurationBlock = new RawConfigurationBlock(&RAWBUFF32[current_size], _configurationBlockVersion);
    }


    /// Finish with the Header
    _Header->setConfigurationBlockSet(1);

    return _Header->setRunSize(current_size + config_block_size);
}

//------------------------------------------------------------------------------
bool RawRunV0::setRunNumber1             (uint32_t w) { return _Header->setRunNumber(w);}
bool RawRunV0::setRunType1               (uint32_t w) { return _Header->setRunType(w);}
bool RawRunV0::setConfigIDX1             (uint32_t w) { return _Header->setConfigIDX(w);}
bool RawRunV0::setPartition1             (uint32_t w) { return _Header->setPartition(w);}
bool RawRunV0::setSubrun1                (uint32_t w) { return _Header->setSubrun(w);}
bool RawRunV0::setRunControlVersion1     (uint32_t w) { return _Header->setRunControlVersion(w);}
bool RawRunV0::setRunControlID1          (uint32_t w) { return _Header->setRunControlID(w);}
bool RawRunV0::setDataLoggerVersion1     (uint32_t w) { return _Header->setDataLoggerVersion(w);}
bool RawRunV0::setDataLoggerID1          (uint32_t w) { return _Header->setDataLoggerID(w);}
bool RawRunV0::setRunStartTime1          (uint32_t w) { return _Header->setRunStartTime(w);}
bool RawRunV0::setSubRunStartTime1       (uint32_t w) { return _Header->setSubRunStartTime(w);}
bool RawRunV0::setSubRunEndTime1         (uint32_t w) { return _Header->setSubRunEndTime(w);}
bool RawRunV0::setRunEndTime1            (uint32_t w) { return _Header->setRunEndTime(w);}
bool RawRunV0::setTriggerCtrlID1         (uint32_t w) { return _Header->setTriggerCtrlID(w);}
bool RawRunV0::setTriggerListIDX1        (uint32_t w) { return _Header->setTriggerListIDX(w);}
bool RawRunV0::setTriggerPrescaleListIDX1(uint32_t w) { return _Header->setTriggerPrescaleListIDX(w);}

//------------------------------------------------------------------------------
bool RawRunV0::setFloatingEventToBufferSource(void* internal_buffer_pointer) const {
            /// If it exists, just set its buffer
            if(_FloatingEvent)
                return _FloatingEvent->setBufferSource(internal_buffer_pointer);

            /// Otherwise, create new Floating MicroSlice pointing to the buffer
            _FloatingEvent = new RawEvent(internal_buffer_pointer, _eventVersion);

            return true;
}


//------------------------------------------------------------------------------
bool RawRunV0::setFloatingEvent1(uint32_t ievent) const{

        /// If it's already the same, nothing needs to be done
        if((int32_t)ievent == _FloatingEventNumber)
            return true;

        /// Pointer that we are going to use
        uint32_t* pointer;

        /// If microslice number is greater than current floating Microslice number, then no need
        /// to loop over all MicroSlices, just start from the current floating MicroSlice
        if((int32_t)ievent > _FloatingEventNumber && _FloatingEventNumber > -1){
            pointer = (uint32_t*)_FloatingEvent->getBuffer();
        }
        else{
            /// Otherwise, start from scratch
            /// First point to the first MicroSlice
            pointer = ((uint32_t*)_Buffer) + _Header->sizeofdata() + _ConfigurationBlock->sizeofdata();


            /// Set floating MicroSlice to pointer
            /// If it doesn't exist, create it
            setFloatingEventToBufferSource(pointer);
            _FloatingEventNumber = 0;

        }// end of setting the first floating MicroSlice


        /// Pointer to the end of the buffer for checks
        const uint32_t* end_of_buffer_pointer = &RAWBUFF32[sizeofdata()];

        /// Start loop over microslices
        for(; _FloatingEventNumber < (int32_t)ievent; ++_FloatingEventNumber){
            pointer += _FloatingEvent->sizeofdata();

            /// Check if the pointer is not valid
            if (pointer >= end_of_buffer_pointer)
                return false;

            _FloatingEvent->setBufferSource(pointer);
        }// end of loop over microslices

        return true;
}

//------------------------------------------------------------------------------
// Printing the Run structure
void RawRunV0::print1() const{
    printf("\n---Run---------------------------------------------size = %u words\n",this->sizeofdata());
    _Header->print();

    std::cout<<"\nRun Data RAW 32bit WORDs (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        this->printWord(i,false);
        printf(" ");
    }

    std::cout<<"\nRun Data RAW 32bit Bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            std::cout<<"\n    ";
            this->printWord(i,true);
    }

}

//------------------------------------------------------------------------------
// Size of the Run _Buffer in words
uint32_t RawRunV0::sizeofdata1() const { return _Header->getRunSize(); }


//------------------------------------------------------------------------------
bool RawRunV0::setFunctionPointers() {

/// Need to define  VERSIONREFERENCE to the current class name
#define     VERSIONREFERENCE RawRunV0


            /// Bind standard methods for all DAQDataFormats:
            ///         init, print, setBufferSource, sizeofdata, getMasksAndCalculateShifts, readData
            /// This macro should always be in every setFunctionPointers() method,
            /// because for sure one of those six have changed
            BIND_STANDARD_METHODS_WITHOUT_SIZEOFDATA

            BIND_FUNCTION_0PAR(getHeader)
            BIND_FUNCTION_0PAR(getTail)
            BIND_FUNCTION_0PAR(getConfigurationBlock)
            BIND_FUNCTION_0PAR(getFloatingEvent)
            BIND_FUNCTION_0PAR(getHeaderConst)
            BIND_FUNCTION_0PAR(getTailConst)
            BIND_FUNCTION_0PAR(getConfigurationBlockConst)
            BIND_FUNCTION_0PAR(getFloatingEventConst)
            BIND_FUNCTION_0PAR(getVersion)
            BIND_FUNCTION_0PAR(getMarkerHi)
            BIND_FUNCTION_0PAR(getMarkerLo)
            BIND_FUNCTION_0PAR(getRunNumber)
            BIND_FUNCTION_0PAR(getRunType)
            BIND_FUNCTION_0PAR(getConfigIDX)
            BIND_FUNCTION_0PAR(getPartition)
            BIND_FUNCTION_0PAR(getSubrun)
            BIND_FUNCTION_0PAR(getConfigurationBlockSet)
            BIND_FUNCTION_0PAR(getRunControlVersion)
            BIND_FUNCTION_0PAR(getRunControlID)
            BIND_FUNCTION_0PAR(getDataLoggerVersion)
            BIND_FUNCTION_0PAR(getDataLoggerID)
            BIND_FUNCTION_0PAR(getRunStartTime)
            BIND_FUNCTION_0PAR(getSubRunStartTime)
            BIND_FUNCTION_0PAR(getSubRunEndTime)
            BIND_FUNCTION_0PAR(getRunEndTime)
            BIND_FUNCTION_0PAR(getTotalEvents)
            BIND_FUNCTION_0PAR(getUniversalEventStartLo)
            BIND_FUNCTION_0PAR(getUniversalEventStartHi)
            BIND_FUNCTION_0PAR(getUniversalEventStart)
            BIND_FUNCTION_0PAR(getUniversalEventEndLo)
            BIND_FUNCTION_0PAR(getUniversalEventEndHi)
            BIND_FUNCTION_0PAR(getUniversalEventEnd)
            BIND_FUNCTION_0PAR(getRunSize)
            BIND_FUNCTION_0PAR(getTriggerVersion)
            BIND_FUNCTION_0PAR(getTriggerCtrlID)
            BIND_FUNCTION_0PAR(getTriggerMC)
            BIND_FUNCTION_0PAR(getVTMod)
            BIND_FUNCTION_0PAR(getValidTriggerTypesLow)
            BIND_FUNCTION_0PAR(getValidTriggerTypesHigh)
            BIND_FUNCTION_0PAR(getValidTriggerTypesHigh2)
            BIND_FUNCTION_0PAR(getTriggerListIDX)
            BIND_FUNCTION_0PAR(getTriggerPrescaleListIDX)

            BIND_FUNCTION_1PAR(setConfigurationBlock1)
            BIND_FUNCTION_1PAR(setConfigurationBlock2)
            BIND_FUNCTION_1PAR(addEvent)
            BIND_FUNCTION_1PAR(setFloatingEvent)
            BIND_FUNCTION_1PAR(setRunNumber)
            BIND_FUNCTION_1PAR(setRunType)
            BIND_FUNCTION_1PAR(setConfigIDX)
            BIND_FUNCTION_1PAR(setPartition)
            BIND_FUNCTION_1PAR(setSubrun)
            BIND_FUNCTION_1PAR(setRunControlVersion)
            BIND_FUNCTION_1PAR(setRunControlID)
            BIND_FUNCTION_1PAR(setDataLoggerVersion)
            BIND_FUNCTION_1PAR(setDataLoggerID)
            BIND_FUNCTION_1PAR(setRunStartTime)
            BIND_FUNCTION_1PAR(setSubRunStartTime)
            BIND_FUNCTION_1PAR(setSubRunEndTime)
            BIND_FUNCTION_1PAR(setRunEndTime)
            BIND_FUNCTION_1PAR(setTriggerCtrlID)
            BIND_FUNCTION_1PAR(setTriggerListIDX)
            BIND_FUNCTION_1PAR(setTriggerPrescaleListIDX)
            BIND_FUNCTION_1PAR(close)
            BIND_FUNCTION_1PAR(figureOutEventSize)
            BIND_FUNCTION_1PAR(figureOutEventSizeWithoutFiguringOutVersion)


#undef VERSIONREFERENCE
            return true;
}

//------------------------------------------------------------------------------
bool RawRunV0::getMasksAndCalculateShifts1(bool) {   return true;}

//------------------------------------------------------------------------------
bool RawRunV0::setBufferSource1(void* pointer){

        _Buffer  = pointer;
        _shouldBufferBeInternal = isInternalBuffer();

        if(isVersionUnknown())return true;

        /// If Header is already defined, then set its buffer source
        /// Otherwise, need to construct it
        if(_Header){
            return _Header->setBufferSource(pointer);
        }

        _Header = new RawRunHeader(_Buffer, _version); ///< version of the DataBlockSliceHeader equals the version of the DataBlockSlice

        /// Now that the header is constructed we need to actually bind sizeofdata
        return bind_sizeofdata();
}
*/