#include <iostream>
#include <iomanip>
#include <vector>
#include <cstdlib>
#include <ctime>
#include <getopt.h>
#include <string>

#include "NovaDAQUtilities/LocationUtils.h"
#include "NovaDatabase/Table.h"

using namespace nova::database;
using namespace std;

string dbHost = "";
string dbName = "";
string dbPort = "";
string dbUser = "";
string dbFname = "";
string detectorName = "";

void PrintUsage()
{
  cout << "Usage: createTableInDB [options]" << endl;
  cout << "options:\n";
  cout << "\t -h (--host) [dB host]" << endl;
  cout << "\t -n (--name) [dB name]" << endl;
  cout << "\t -p (--port) [dB port]" << endl;
  cout << "\t -u (--user) [dB user]" << endl;
  cout << "\t -d (--detector) [detector name, REQUIRED]" << endl;
  cout << "\t -f (--file) [XML table def. file, REQUIRED]" << endl;
}

//------------------------------------------------------------
bool ParseCLArgs(int argc, char* argv[])
{
  if (argc == 1) {
    PrintUsage();
    exit(0);
  }

  struct option long_options[] = {
    {"host",      1, 0, 'h'},
    {"port",      1, 0, 'p'},
    {"name",      1, 0, 'n'},
    {"detector",  1, 0, 'd'},
    {"file",      1, 0, 'f'},
    {"user",      1, 0, 'u'},
    {0,0,0,0}
  };
  
  while (1) {
    int optindx;
    int c = getopt_long(argc,argv,"h:p:n:d:f:u:",long_options,&optindx);
        
    if (c==-1) break;
    
    switch(c) {
    case 'h':
      dbHost = optarg;
      break;
    case 'p':
      dbPort = optarg;
      break;
    case 'n':
      dbName = optarg;
      break;
    case 'd':
      detectorName = optarg;
      break;
    case 'f':
      dbFname = optarg;
      break;
    case 'u':
      dbUser = optarg;
      break;
    default:
      break;
    }
    
  }

  if (optind<argc) {
    std::cerr << "Unrecognized argument." << std::endl;
    PrintUsage();
    exit(0);
  }

  char* s;
  if (dbHost == "") {
    s = getenv("NOVADBHOST");
    if (s)
      dbHost = s;
    if (dbHost == "") {
      std::cerr << "No value set for NOVADBHOST.  Aborting." << std::endl;
      exit(0);
    }
  }
  if (dbPort == "") {
    s = getenv("NOVADBPORT");
    if (s)
      dbPort = s;
    if (dbPort == "") {
      std::cerr << "No value set for NOVADBPORT.  Aborting." << std::endl;
      exit(0);
    }
  }
  if (dbName == "") {
    s = getenv("NOVADBNAME");
    if (s)
      dbName = s;
    if (dbName == "") {
      std::cerr << "No value set for NOVADBNAME.  Aborting." << std::endl;
      exit(0);
    }
  }
  if (dbUser == "") {
    s = getenv("NOVADBUSER");
    if (s)
      dbUser = s;
    if (dbUser == "") {
      std::cerr << "No value set for NOVADBUSER.  Aborting." << std::endl;
      exit(0);
    }
  }
  
  return true;
}

//------------------------------------------------------------

int main(int argc, char *argv[])
{
  ParseCLArgs(argc, argv);

  Table* dbt;
  try {
    dbt = new Table(dbFname);
  }
  catch (std::runtime_error& e) {
    std::cerr << e.what() << std::endl;
    exit(1);
  }

  if (!dbt->SetDetector(detectorName)) {
    exit(1);
  }

  for (unsigned int k=0; k<detectorName.length(); ++k)
    detectorName[k] = tolower(detectorName[k]);

  dbt->SetIgnoreEnvVar(true);

  dbt->SetDBHost(dbHost);
  dbt->SetDBName(dbName);
  dbt->SetDBPort(dbPort);
  dbt->SetUser(dbUser);

  if (! dbt->GetConnection()) {
    std::cerr << "Could not connect to the database.  Exiting..." << std::endl;
    return 1;
  }

  if (dbt->ExistsInDB()) {
    std::cerr << dbt->Name() << " already exists in the database." 
	      << std::endl;
    dbt->CloseConnection();
    return 0;
  }

  cerr << "Creating dB table " << dbt->Name() << " in  database." << endl;

  string cmd;
  PGresult* res;

  // set role to nova_admin to create the table
  cmd = "SET ROLE nova_admin";
  if (dbt->ExecuteSQL(cmd,res)) 
    if (PQresultStatus(res) != PGRES_COMMAND_OK) {
      cerr << cmd << " failed: ";
      dbt->PrintPQErrorMsg();
      dbt->CloseConnection();
      return 1;
    }
  
  vector<const Column*> serialList(0);

  cmd = "CREATE TABLE " + dbt->Schema() + "." + dbt->Name() + "(";
  for (int i=0; i<dbt->NCol(); ++i) {
    if (i>0) cmd += ",";

    const Column* col = dbt->GetCol(i);
    const dBColumn_t* xCol = dbt->XMLCol[col->Name()];

    cmd += col->Name() + " ";      
    string ctype = col->Type();
    if (ctype == "int")
      cmd += "integer";
    else if (ctype == "long" || ctype == "bigint")
      cmd += "bigint";
    else if (ctype == "short")
      cmd += "smallint";
    else if (ctype == "float")
      cmd += "real";
    else if (ctype == "double")
      cmd += "double precision";
    else if (ctype == "bool")
      cmd += "boolean";
    else if (ctype == "timestamp")
      cmd += "timestamp without time zone";
    else if (ctype == "date")
      cmd += "date";
    else if (ctype == "string" || ctype== "text")
      cmd += "text";
    else if (ctype.substr(0,7) == "varchar") {
      cmd += "varchar(" + ctype.substr(7) + ")";
    }
    else if (ctype == "autoincr") {
      cmd += "serial";
      serialList.push_back(col);      
    }
    else {
      cerr << "createTableInDB: Unknown column type \"" << ctype 
	   << "\".  Aborting..." << endl;
      dbt->CloseConnection();
      return 2;
    }

    if (xCol->hasFKeyTable()) {
      cmd += " REFERENCES " + dbt->Schema() + "." + xCol->getFKeyTable();
      if (xCol->hasFKeyColumn()) {
	cmd += " (" + xCol->getFKeyColumn() + ")";
      }
      cmd += " ON DELETE RESTRICT ON UPDATE RESTRICT";
    }
    
    if (! col->CanBeNull())
      cmd += " NOT NULL ";

  }

  if (dbt->HasPKey()) {
    // add code for PRIMARY KEY (CONSTRAINT)
    cmd += ",PRIMARY KEY (";
    cmd += dbt->GetPKeyCol(0)->Name();
    if (dbt->GetNPKeyCol() > 1) {
      for (int j=1; j<dbt->GetNPKeyCol(); ++j) {
	cmd += ",";
	cmd += dbt->GetPKeyCol(j)->Name();
      }
    }
    cmd += ")";
    
  }

  // add code for UNIQUE CONSTRAINT
  if (dbt->GetNUnique() > 0) {
    for (int i=0; i<dbt->GetNUnique(); ++i) {
      cmd += ", UNIQUE (";
      const dBRow_t* uq = dbt->GetUniqueRow(i);
      cmd += uq->getDBCol(0).getName();
      if (uq->getDBColCount() > 1)
	for (int j=1; j<uq->getDBColCount(); ++j) {
	  cmd += ",";
	  cmd += uq->getDBCol(j).getName();
	}
      cmd += ")";
    }
  }
  
  // add code for CHECK CONSTRAINT
  if (dbt->GetNCheck() > 0) 
    for (int i=0; i<dbt->GetNCheck(); ++i) {
      cmd += ", CHECK (";           
      cmd += dbt->GetCheckConstraint(i);
      cmd += ")";
    }
  
  cmd += ") ";
  
  cerr << "Table::CreateTableInDB: Executing PGSQL command: \n\t" 
       << cmd << endl;

  // now execute command for real
  PQclear(res);

  if (dbt->ExecuteSQL(cmd,res)) 
    if (PQresultStatus(res) != PGRES_COMMAND_OK) {
      cerr << cmd << " failed: ";
      dbt->PrintPQErrorMsg();
      dbt->CloseConnection();
      return 1;
    }
  
  // now set privs
  PQclear(res);  
  cmd = "GRANT SELECT ON TABLE ";
  cmd += dbt->Schema() + "." + dbt->Name();
  cmd += " TO public";
  if (dbt->ExecuteSQL(cmd,res)) 
    if (PQresultStatus(res) != PGRES_COMMAND_OK) {
      cerr << cmd << " failed: ";
      dbt->PrintPQErrorMsg();
      dbt->CloseConnection();
      return 1;
    }

  PQclear(res);
  cmd = "GRANT ALL ON TABLE ";
  cmd += dbt->Schema() + "." + dbt->Name();
  cmd += " TO nova_writer";
  if (dbt->ExecuteSQL(cmd,res)) 
    if (PQresultStatus(res) != PGRES_COMMAND_OK) {
      cerr << cmd << " failed: ";
      dbt->PrintPQErrorMsg();
      dbt->CloseConnection();
      return 1;
    }

  PQclear(res);
  
  dbt->CloseConnection();

  return 0;
}

